/* Gravis Ultrasound MIDI port test program */
/* Copyright 1993 Dave Perry */
/* This program may be used for non-commercial purposes only */

#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#define CONTROL 0x100      /* Control port offset		 */
#define DATA    0x101      /* Data port offset			 */
#define BUFSIZE 8000        /* Ring buffer				 */
#define PIC_MASK 0x21      /* 8259 interrupt mask port	 */
#define PIC_MASK2 0xa1     /* Secondary 8259 mask port	 */
#define MPU_DSR  0x02
#define MPU_DRR  0x01
#define MPU_UART      0x3F                             /*  Set UART mode  */
#define MPU_RESET     0xFF                 /*  Reset to default settings  */
#define MPU_WAIT      0xFF      /*  Wait this many ms for command to act  */
#define MPU_ACK       0xFE      /*  MPU acknowledges a command with this  */
#define MPU_TIMEOUT   0xFF     /*  To flag MPU_RETRIES has been exceeded  */
#define MPU_RETRIES   0xFF      /*  Times to try read/write from/to port  */

#define MIDI_SYSEX    0xF0      /*  MIDI: Start System Exclusive message  */
#define MIDI_EOX      0xF7        /*  MIDI: End System Exclusive message  */

unsigned char buffer[BUFSIZE];
unsigned char *head = buffer;
unsigned char *tail = buffer;

unsigned int base,midi_irq; /* Card base address */

void interrupt ( *old_isr)(...);

/*
status byte   meaning        data bytes

0x80-0x8f     note off       2 - 1 byte pitch, followed by 1 byte velocity
0x90-0x9f     note on        2 - 1 byte pitch, followed by 1 byte velocity
0xa0-0xaf     key pressure   2 - 1 byte pitch, 1 byte pressure (after-touch)
0xb0-0xbf     parameter      2 - 1 byte parameter number, 1 byte setting
0xc0-0xcf     program        1 byte program selected
0xd0-0xdf     chan. pressure 1 byte channel pressure (after-touch)
0xe0-0xef     pitch wheel    2 bytes gives a 14 bit value, least significant
				   7 bits first
*/

/* ---------------------------------------------- */
/*  mpu_put_byte - write a byte to the MPU        */
/*  Returns: ??? (STATPORT) ??? on success        */
/*           MPU_TIMEOUT if MPU_RETRIES exceeded  */
/* ---------------------------------------------- */

int mpu_put_byte(int byte)
{
   int status, c;

   status = inp(base+ CONTROL);
   c = 0;
   while (!(inportb(base + CONTROL) & 0x02))
;
/*  status = inp(base + CONTROL);
   if (c >= MPU_RETRIES)
      status = MPU_TIMEOUT;
   else {
      delay(1);*/
      outp(base + DATA, byte);
      status = inp(base + CONTROL);
/* }*/
   return(status);
}  /*  end mpu_put_byte  */


void
send_eoi()
{
	asm mov	al,0bh;		/* read in-service register from */
	asm out	0a0h,al;	/* secondary 8259 */
	asm nop;			/* settling delay */
	asm nop
	asm nop
	asm in	al,0a0h;    /* get it */
	asm or	al,al;      /* Any bits set? */
	asm jz	lab1;       /* nope, not a secondary interrupt */
/*	delay(1);*/
	asm mov	al,20h;     /* Get EOI instruction */
	asm out	0a0h,al;    /* Secondary 8259 */
lab1:	asm mov	al,20h; /* 8259 end-of-interrupt command */
	asm out	20h,al;     /*  Primary 8259 */
}


/* MIDI input interrupt handler */
void interrupt
midi_isr(...)
{
	static cnt = 0;
	*head = inportb(base+DATA);  /* Grab the data */
	asm cli;
	if(cnt == BUFSIZE - 1) {/* Adjust the buffer pointer */
		cnt = 0;
		head = buffer;
	} else {
		++head;
		++cnt;
	}
	asm sti;
	send_eoi();
}

void
hook_irq(unsigned char irq)
{
	unsigned char abyte,int_mask,vector;
	unsigned int pic_mask; /* Address of PIC mask */

	if(irq < 8){
		vector = irq + 8;
		pic_mask = PIC_MASK;
		int_mask = 1 << irq;
	}
	else {
		vector = irq + 0x70 - 8;
		pic_mask = PIC_MASK2;
		int_mask = 1 << (irq - 8);
	}


	/* save the existing interrupt vector */
	old_isr = getvect(vector);

	/* Install our interrupt vector */
	setvect(vector,midi_isr);

	/* Enable interrupts on the MIDI UART */
	outportb(base+CONTROL,0x95);

	/* Set up the 8259 PIC chip */
	abyte = inportb(pic_mask); /* Read 8259 mask */
	abyte &= ~int_mask;        /* Clear mask for desired IRQ */
	outportb(pic_mask,abyte);

}

void
unhook_irq(unsigned char irq)
{

	unsigned char abyte, int_mask, vector;
	unsigned int pic_mask;

	if(irq < 8){
		vector = irq + 8;
		pic_mask = PIC_MASK;
		int_mask = 1 << irq;
	}
	else {
		vector = irq + 0x70 - 8;
		pic_mask = PIC_MASK2;
		int_mask = 1 << (irq - 8);
	}

	/* Mask the interrupt we hooked */
	abyte = inportb(pic_mask); /* Read 8259 mask */
	abyte |= int_mask;         /* Set mask for desired IRQ */
	outportb(pic_mask,abyte);

	/* Restore old interrupt handler */
	setvect(vector,midi_isr);
}

void EnableMidi()
{
	int abyte;
	char *envstring;
	unsigned int dummy;

	envstring = getenv("ULTRASND");
	if(envstring == NULL) {
		printf("ULTRASND environment string not found\n");
		printf("Check your Ultrasound setup\n");
		exit(0);
	}
	sscanf(envstring,"%x,%d,%d,%d,%d",
		&base,&dummy,&dummy,&dummy,&midi_irq);
	hook_irq(midi_irq);
}

void MidiShutDown()
{
	unhook_irq(midi_irq);
}

void MidiXmit(unsigned char data)
{
 int temp;
temp= mpu_put_byte((int)data);
delay(1);

}

void MidiREnable(void)
{
   outp(base + CONTROL,0x03);
   delay(5);
   outp(base + CONTROL,0x00);
   hook_irq(midi_irq);
}

void MidiRDisable(void)
{
   unhook_irq(midi_irq);
}
unsigned char MidiRecv(void)
{
	static cnt = 0,status;
	unsigned char retval;
	while (head == tail)
	;
	asm cli;                /* Ints off for pointer manipulation */
	status = inp(base+CONTROL);
	if (head != tail)
	{
		retval = *tail++;
		cnt++;
		if(cnt == BUFSIZE) {
			cnt = 0;
			tail = buffer;
		}
	}
	asm sti;
	return(retval);
}

void FlushMidi(void)
{
}


